@@ -105,47 +105,45 @@ mapi_folder_search_by_uids (CamelFolder *folder, const gchar *expression, GPtrAr
 	return matches;
 }
 
-static gboolean
-mapi_refresh_info(CamelFolder *folder, CamelException *ex)
+static void
+update_store_summary (CamelFolder *folder, CamelException *ex)
 {
 	CamelStore *parent_store;
+	CamelStoreSummary *store_summary;
 	CamelStoreInfo *si;
 	const gchar *full_name;
 
 	full_name = camel_folder_get_full_name (folder);
 	parent_store = camel_folder_get_parent_store (folder);
+	store_summary = (CamelStoreSummary *)((CamelMapiStore *)parent_store)->summary;
 
-	/*
-	 * Checking for the summary->time_string here since the first the a
-	 * user views a folder, the read cursor is in progress, and the getQM
-	 * should not interfere with the process
-	 */
-	//	if (summary->time_string && (strlen (summary->time_string) > 0))  {
-	if (1) {
-		mapi_refresh_folder(folder, ex);
-		si = camel_store_summary_path ((CamelStoreSummary *)((CamelMapiStore *)parent_store)->summary, full_name);
+	si = camel_store_summary_path (store_summary, full_name);
 
-		if (si) {
-			guint32 unread, total;
+	if (si) {
+		guint32 unread, total;
 
-			unread = folder->summary->unread_count;
-			total = camel_folder_summary_count (folder->summary);
-			if (si->total != total || si->unread != unread) {
-				si->total = total;
-				si->unread = unread;
-				camel_store_summary_touch ((CamelStoreSummary *)((CamelMapiStore *)parent_store)->summary);
-			}
-			camel_store_summary_info_free ((CamelStoreSummary *)((CamelMapiStore *)parent_store)->summary, si);
+		unread = folder->summary->unread_count;
+		total = camel_folder_summary_count (folder->summary);
+
+		if (si->total != total || si->unread != unread) {
+			si->total = total;
+			si->unread = unread;
+			camel_store_summary_touch (store_summary);
 		}
-		camel_folder_summary_save_to_db (folder->summary, ex);
-		camel_store_summary_save ((CamelStoreSummary *)((CamelMapiStore *)parent_store)->summary);
-	} else {
-		/* We probably could not get the messages the first time. (get_folder) failed???!
-		 * so do a get_folder again. And hope that it works
-		 */
-		g_print("Reloading folder...something wrong with the summary....\n");
+		camel_store_summary_info_free (store_summary, si);
 	}
-	//#endif
+	camel_folder_summary_save_to_db (folder->summary, ex);
+	camel_store_summary_save (store_summary);
+}
+
+static gboolean
+mapi_refresh_info (CamelFolder *folder, CamelException *ex)
+{
+	mapi_refresh_folder (folder, ex);
+	if (camel_exception_is_set (ex))
+		return FALSE;
+
+	update_store_summary (folder, ex);
 
 	return !camel_exception_is_set (ex);
 }
@@ -157,7 +155,7 @@ fetch_items_summary_cb (FetchItemsCallbackData *item_data, gpointer data)
 
 	GSList **slist = &(fi_data->items_list);
 
-	long *flags;
+	long *flags = NULL;
 	struct FILETIME *delivery_date = NULL;
 	struct FILETIME *last_modification_time = NULL;
 	struct timeval item_modification_time = { 0 };
@@ -520,18 +518,6 @@ mapi_update_cache (CamelFolder *folder, GSList *list, CamelFolderChangeInfo **ch
 	g_string_free (str, TRUE);
 }
 
-static void
-mapi_sync_summary (CamelFolder *folder, CamelException *ex)
-{
-	CamelStore *parent_store;
-
-	parent_store = camel_folder_get_parent_store (folder);
-
-	camel_folder_summary_save_to_db (folder->summary, ex);
-	camel_store_summary_touch ((CamelStoreSummary *)((CamelMapiStore *)parent_store)->summary);
-	camel_store_summary_save ((CamelStoreSummary *)((CamelMapiStore *)parent_store)->summary);
-}
-
 static void
 mapi_utils_do_flags_diff (flags_diff_t *diff, guint32 old, guint32 _new)
 {
@@ -550,11 +536,17 @@ struct mapi_update_deleted_msg {
 static gboolean
 deleted_items_sync_cb (FetchItemsCallbackData *item_data, gpointer data)
 {
-	GSList **uid_list = (GSList **) data;
+	guint32 msg_flags = CAMEL_MESSAGE_FOLDER_FLAGGED; /* to not have 0 in the hash table */
+	GHashTable *uids = data;
 	gchar *msg_uid = exchange_mapi_util_mapi_ids_to_uid (item_data->fid,
 							     item_data->mid);
 
-	*uid_list = g_slist_prepend (*uid_list, msg_uid);
+	if ((item_data->msg_flags & MSGFLAG_READ) != 0)
+		msg_flags |= CAMEL_MESSAGE_SEEN;
+	if ((item_data->msg_flags & MSGFLAG_HASATTACH) != 0)
+		msg_flags |= CAMEL_MESSAGE_ATTACHMENTS;
+
+	g_hash_table_insert (uids, msg_uid, GINT_TO_POINTER (msg_flags));
 
 	/* Progress update */
 	if (item_data->total > 0)
@@ -567,18 +559,6 @@ deleted_items_sync_cb (FetchItemsCallbackData *item_data, gpointer data)
 	return TRUE;
 }
 
-static gboolean
-mapi_camel_get_last_modif_list (ExchangeMapiConnection *conn, mapi_id_t fid, TALLOC_CTX *mem_ctx, struct SPropTagArray *props, gpointer data)
-{
-	static const uint32_t prop_list[] = {
-		PR_LAST_MODIFICATION_TIME
-	};
-
-	g_return_val_if_fail (props != NULL, FALSE);
-
-	return exchange_mapi_utils_add_props_to_props_array (mem_ctx, props, prop_list, G_N_ELEMENTS (prop_list));
-}
-
 static void
 mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 {
@@ -591,8 +571,9 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 	CamelStore *parent_store;
 
 	guint32 index, count, options = 0;
-	GSList *server_uid_list = NULL;
+	GHashTable *server_messages = NULL;
 	const gchar *uid = NULL;
+	gboolean flags_changed = FALSE;
 
 	parent_store = camel_folder_get_parent_store (m->folder);
 
@@ -614,10 +595,12 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 	if (mapi_folder->type & CAMEL_MAPI_FOLDER_PUBLIC)
 		options |= MAPI_OPTIONS_USE_PFSTORE;
 
+	server_messages = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+
 	/*Get the UID list from server.*/
 	exchange_mapi_connection_fetch_items  (camel_mapi_store_get_exchange_connection (mapi_store), m->folder_id, NULL, NULL,
-					       mapi_camel_get_last_modif_list, NULL,
-					       deleted_items_sync_cb, &server_uid_list,
+					       NULL, NULL,
+					       deleted_items_sync_cb, server_messages,
 					       options | MAPI_OPTIONS_DONT_OPEN_MESSAGE);
 
 	camel_operation_end (NULL);
@@ -625,8 +608,10 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 	camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	/* Check if we have to stop */
-	if (camel_operation_cancel_check(NULL))
+	if (camel_operation_cancel_check(NULL)) {
+		g_hash_table_destroy (server_messages);
 		return;
+	}
 
 	changes = camel_folder_change_info_new ();
 
@@ -637,34 +622,47 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 
 	/* Iterate over cache and check if the UID is in server*/
 	for (index = 0; index < count; index++) {
-		GSList *tmp_list_item = NULL;
+		guint32 msg_flags;
 
 		/* Iterate in a reverse order, thus removal will not hurt */
 		info = camel_folder_summary_index (m->folder->summary, count - index - 1);
 		if (!info) continue; /*This is bad. *Should* not happen*/
 
 		uid = camel_message_info_uid (info);
-
-		if (server_uid_list) {
-			/* TODO : Find a better way and avoid this linear search */
-			tmp_list_item = g_slist_find_custom (server_uid_list, (gconstpointer) uid,
-						  (GCompareFunc) g_strcmp0);
+		if (!uid) {
+			camel_message_info_free (info);
+			continue;
 		}
 
+		msg_flags = GPOINTER_TO_INT (g_hash_table_lookup (server_messages, uid));
+
 		/* If it is not in server list, clean our cache */
-		if ((!tmp_list_item || !tmp_list_item->data) && uid) {
+		if (!msg_flags) {
 			CAMEL_MAPI_FOLDER_REC_LOCK (m->folder, cache_lock);
 			camel_folder_summary_remove_uid (m->folder->summary, uid);
 			camel_data_cache_remove (mapi_folder->cache, "cache", uid, NULL);
 			camel_folder_change_info_remove_uid (changes, uid);
 			CAMEL_MAPI_FOLDER_REC_UNLOCK (m->folder, cache_lock);
+		} else {
+			CamelMapiMessageInfo *mapi_info = (CamelMapiMessageInfo *) info;
+
+			msg_flags = msg_flags & (~CAMEL_MESSAGE_FOLDER_FLAGGED);
+			if (mapi_info->server_flags != msg_flags) {
+				mapi_info->server_flags = msg_flags;
+				camel_message_info_set_flags (info, msg_flags, CAMEL_MESSAGE_SEEN | CAMEL_MESSAGE_ATTACHMENTS);
+				camel_folder_change_info_change_uid (changes, uid);
+				flags_changed = TRUE;
+			}
 		}
 
+		camel_message_info_free (info);
+
 		/* Progress update */
 		camel_operation_progress (NULL, (index * 100)/count); /* ;-) */
 
 		/* Check if we have to stop */
 		if (camel_operation_cancel_check(NULL)) {
+			g_hash_table_destroy (server_messages);
 			if (camel_folder_change_info_changed (changes))
 				camel_folder_changed (m->folder, changes);
 			camel_folder_change_info_free (changes);
@@ -674,15 +672,16 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 
 	camel_operation_end (NULL);
 
-	if (camel_folder_change_info_changed (changes))
+	if (camel_folder_change_info_changed (changes)) {
+		if (flags_changed)
+			camel_mapi_summary_update_store_info_counts (CAMEL_MAPI_SUMMARY (CAMEL_FOLDER (mapi_folder)->summary));
 		camel_folder_changed (m->folder, changes);
+	}
 	camel_folder_change_info_free (changes);
 
-	m->need_refresh = camel_folder_summary_count (m->folder->summary) != g_slist_length (server_uid_list);
+	m->need_refresh = camel_folder_summary_count (m->folder->summary) != g_hash_table_size (server_messages);
 
-	/* Discard server uid list */
-	g_slist_foreach (server_uid_list, (GFunc) g_free, NULL);
-	g_slist_free (server_uid_list);
+	g_hash_table_destroy (server_messages);
 }
 
 static void
@@ -728,16 +727,14 @@ mapi_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 	CamelMapiMessageInfo *mapi_info = NULL;
 	CamelStore *parent_store;
 
-	GSList *read_items = NULL, *unread_items = NULL, *to_free = NULL;
+	GSList *read_items = NULL, *unread_items = NULL, *to_free = NULL, *junk_items = NULL, *deleted_items = NULL, *l;
 	flags_diff_t diff, unset_flags;
 	const gchar *folder_id;
 	const gchar *full_name;
 	mapi_id_t fid, deleted_items_fid;
 	gint count, i;
 	guint32 options =0;
-
-	GSList *deleted_items, *deleted_head;
-	deleted_items = deleted_head = NULL;
+	gboolean is_junk_folder;
 
 	full_name = camel_folder_get_full_name (folder);
 	parent_store = camel_folder_get_parent_store (folder);
@@ -747,7 +744,7 @@ mapi_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 
 	if (((CamelOfflineStore *) mapi_store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL ||
 			((CamelService *)mapi_store)->status == CAMEL_SERVICE_DISCONNECTED) {
-		mapi_sync_summary (folder, ex);
+		update_store_summary (folder, ex);
 		return !camel_exception_is_set (ex);
 	}
 
@@ -765,6 +762,8 @@ mapi_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 	}
 	camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
+	is_junk_folder = (mapi_folder->type & CAMEL_FOLDER_TYPE_MASK) == CAMEL_FOLDER_TYPE_JUNK;
+
 	count = camel_folder_summary_count (folder->summary);
 	CAMEL_MAPI_FOLDER_REC_LOCK (folder, cache_lock);
 	for (i=0; i < count; i++) {
@@ -795,22 +794,13 @@ mapi_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 				camel_message_info_free (info);
 				g_free (mid);
 				continue;
-			} else {
-				if (diff.bits & CAMEL_MESSAGE_DELETED) {
-					if (diff.bits & CAMEL_MESSAGE_SEEN) {
-						read_items = g_slist_prepend (read_items, mid);
-						used = TRUE;
-					}
-					if (deleted_items) {
-						deleted_items = g_slist_prepend (deleted_items, mid);
-						used = TRUE;
-					} else {
-						g_slist_free (deleted_head);
-						deleted_head = NULL;
-						deleted_head = deleted_items = g_slist_prepend (deleted_items, mid);
-						used = TRUE;
-					}
-				}
+			}
+			if (diff.bits & CAMEL_MESSAGE_DELETED) {
+				deleted_items = g_slist_prepend (deleted_items, mid);
+				used = TRUE;
+			} else if (!is_junk_folder && (diff.bits & CAMEL_MESSAGE_JUNK) != 0) {
+				junk_items = g_slist_prepend (junk_items, mid);
+				used = TRUE;
 			}
 
 			if (diff.bits & CAMEL_MESSAGE_SEEN) {
@@ -841,45 +831,56 @@ mapi_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 
 	if (read_items) {
 		camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-		exchange_mapi_connection_set_flags (camel_mapi_store_get_exchange_connection (mapi_store), 0, fid, read_items, 0, options);
+		exchange_mapi_connection_set_flags (camel_mapi_store_get_exchange_connection (mapi_store), 0, fid, options, read_items, 0);
 		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-		g_slist_free (read_items);
 	}
 
 	if (unread_items) {
 		camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-		exchange_mapi_connection_set_flags (camel_mapi_store_get_exchange_connection (mapi_store), 0, fid, unread_items, CLEAR_READ_FLAG, options);
+		exchange_mapi_connection_set_flags (camel_mapi_store_get_exchange_connection (mapi_store), 0, fid, options, unread_items, CLEAR_READ_FLAG);
 		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-		g_slist_free (read_items);
 	}
 
 	/* Remove messages from server*/
 	if (deleted_items) {
 		camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 		if ((mapi_folder->type & CAMEL_FOLDER_TYPE_MASK) == CAMEL_FOLDER_TYPE_TRASH) {
-			exchange_mapi_connection_remove_items (camel_mapi_store_get_exchange_connection (mapi_store), 0, fid, deleted_items);
+			exchange_mapi_connection_remove_items (camel_mapi_store_get_exchange_connection (mapi_store), 0, fid, options, deleted_items);
 		} else {
 			exchange_mapi_util_mapi_id_from_string (camel_mapi_store_system_folder_fid (mapi_store, olFolderDeletedItems), &deleted_items_fid);
-			exchange_mapi_connection_move_items (camel_mapi_store_get_exchange_connection (mapi_store), fid, deleted_items_fid, deleted_items);
+			exchange_mapi_connection_move_items (camel_mapi_store_get_exchange_connection (mapi_store), fid, options, deleted_items_fid, 0, deleted_items);
 		}
 
 		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 	}
 
+	
+	if (junk_items) {
+		mapi_id_t junk_fid = 0;
+
+		camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		exchange_mapi_util_mapi_id_from_string (camel_mapi_store_system_folder_fid (mapi_store, olFolderJunk), &junk_fid);
+		exchange_mapi_connection_move_items (camel_mapi_store_get_exchange_connection (mapi_store), fid, options, junk_fid, 0, junk_items);
+		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+
+		/* in junk_items are only emails which are not deleted */
+		deleted_items = g_slist_concat (deleted_items, g_slist_copy (junk_items));
+	}
+
 	/*Remove messages from local cache*/
-	while (deleted_items) {
-		gchar * deleted_msg_uid = g_strdup_printf ("%016" G_GINT64_MODIFIER "X%016" G_GINT64_MODIFIER "X", fid, *(mapi_id_t *)deleted_items->data);
+	for (l = deleted_items; l; l = l->next) {
+		gchar * deleted_msg_uid = g_strdup_printf ("%016" G_GINT64_MODIFIER "X%016" G_GINT64_MODIFIER "X", fid, *(mapi_id_t *)l->data);
 
 		CAMEL_MAPI_FOLDER_REC_LOCK (folder, cache_lock);
 		camel_folder_summary_remove_uid (folder->summary, deleted_msg_uid);
 		camel_data_cache_remove(mapi_folder->cache, "cache", deleted_msg_uid, NULL);
 		CAMEL_MAPI_FOLDER_REC_UNLOCK (folder, cache_lock);
-
-		deleted_items = g_slist_next (deleted_items);
 	}
 
+	g_slist_free (read_items);
 	g_slist_free (unread_items);
-	g_slist_free (deleted_head);
+	g_slist_free (deleted_items);
+	g_slist_free (junk_items);
 
 	g_slist_foreach (to_free, (GFunc) g_free, NULL);
 	g_slist_free (to_free);
@@ -889,7 +890,7 @@ mapi_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 	}
 
 	camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-	mapi_sync_summary (folder, ex);
+	update_store_summary (folder, ex);
 	camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	return !camel_exception_is_set (ex);
@@ -1066,7 +1067,7 @@ mapi_refresh_folder(CamelFolder *folder, CamelException *ex)
 		mapi_summary->sync_time_stamp = g_time_val_to_iso8601 (&fetch_data->last_modification_time);
 
 		camel_folder_summary_touch (folder->summary);
-		mapi_sync_summary (folder, ex);
+		update_store_summary (folder, ex);
 
 		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 		is_locked = FALSE;
@@ -1322,7 +1323,7 @@ mapi_expunge (CamelFolder *folder, CamelException *ex)
 
 	if ((mapi_folder->type & CAMEL_FOLDER_TYPE_MASK) == CAMEL_FOLDER_TYPE_TRASH) {
 		camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-		status = exchange_mapi_connection_empty_folder (camel_mapi_store_get_exchange_connection (mapi_store), fid);
+		status = exchange_mapi_connection_empty_folder (camel_mapi_store_get_exchange_connection (mapi_store), fid, 0);
 		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 		if (status) {
@@ -1366,7 +1367,7 @@ mapi_expunge (CamelFolder *folder, CamelException *ex)
 	if (deleted_items) {
 		camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
-		status = exchange_mapi_connection_remove_items (camel_mapi_store_get_exchange_connection (mapi_store), 0, fid, deleted_items);
+		status = exchange_mapi_connection_remove_items (camel_mapi_store_get_exchange_connection (mapi_store), 0, fid, 0, deleted_items);
 
 		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
@@ -1403,6 +1404,7 @@ mapi_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 		gboolean delete_originals, CamelException *ex)
 {
 	mapi_id_t src_fid, dest_fid;
+	guint32 src_fid_options, dest_fid_options;
 
 	CamelOfflineStore *offline;
 	CamelMapiStore *mapi_store;
@@ -1413,9 +1415,21 @@ mapi_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 	const gchar *source_full_name;
 	const gchar *destination_full_name;
 	gint i = 0;
-
 	GSList *src_msg_ids = NULL;
 
+	if (!CAMEL_IS_MAPI_FOLDER (source) || !CAMEL_IS_MAPI_FOLDER (destination) ||
+	    (CAMEL_MAPI_FOLDER (source)->type & CAMEL_MAPI_FOLDER_PUBLIC) != 0 ||
+	    (CAMEL_MAPI_FOLDER (destination)->type & CAMEL_MAPI_FOLDER_PUBLIC) != 0) {
+		CamelFolderClass *folder_class;
+
+		/* because cannot use MAPI to copy/move messages with public folders,
+		   thus fallback to per-message copy/move */
+		folder_class = CAMEL_FOLDER_CLASS (camel_mapi_folder_parent_class);
+		return folder_class->transfer_messages_to (
+			source, uids, destination, transferred_uids,
+			delete_originals, ex);
+	}
+
 	source_full_name = camel_folder_get_full_name (source);
 	source_parent_store = camel_folder_get_parent_store (source);
 
@@ -1427,13 +1441,15 @@ mapi_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 
 	/* check for offline operation */
 	if (offline->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
-		return TRUE;
+		return FALSE;
 
 	folder_id =  camel_mapi_store_folder_id_lookup (mapi_store, source_full_name);
 	exchange_mapi_util_mapi_id_from_string (folder_id, &src_fid);
+	src_fid_options = (CAMEL_MAPI_FOLDER (source)->type & CAMEL_MAPI_FOLDER_PUBLIC) != 0 ? MAPI_OPTIONS_USE_PFSTORE : 0;
 
 	folder_id = camel_mapi_store_folder_id_lookup (mapi_store, destination_full_name);
 	exchange_mapi_util_mapi_id_from_string (folder_id, &dest_fid);
+	dest_fid_options = (CAMEL_MAPI_FOLDER (destination)->type & CAMEL_MAPI_FOLDER_PUBLIC) != 0 ? MAPI_OPTIONS_USE_PFSTORE : 0;
 
 	for (i=0; i < uids->len; i++) {
 		mapi_id_t *mid = g_new0 (mapi_id_t, 1); /* FIXME : */
@@ -1444,8 +1460,11 @@ mapi_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 	}
 
 	if (delete_originals) {
-		if (!exchange_mapi_connection_move_items (camel_mapi_store_get_exchange_connection (mapi_store), src_fid, dest_fid, src_msg_ids)) {
-			//TODO : Set exception.
+		enum MAPISTATUS retval = exchange_mapi_connection_move_items (camel_mapi_store_get_exchange_connection (mapi_store), src_fid, src_fid_options, dest_fid, dest_fid_options, src_msg_ids);
+		if (retval != MAPI_E_SUCCESS) {
+			const gchar *err_str = mapi_get_errstr (retval);
+
+			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, err_str ? err_str : _("Unknown error"));
 		} else {
 			changes = camel_folder_change_info_new ();
 
@@ -1458,8 +1477,11 @@ mapi_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 
 		}
 	} else {
-		if (!exchange_mapi_connection_copy_items (camel_mapi_store_get_exchange_connection (mapi_store), src_fid, dest_fid, src_msg_ids)) {
-			//TODO : Set exception.
+		enum MAPISTATUS retval = exchange_mapi_connection_copy_items (camel_mapi_store_get_exchange_connection (mapi_store), src_fid, src_fid_options, dest_fid, dest_fid_options, src_msg_ids);
+		if (retval != MAPI_E_SUCCESS) {
+			const gchar *err_str = mapi_get_errstr (retval);
+
+			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, err_str ? err_str : _("Unknown error"));
 		}
 	}
 
@@ -1634,7 +1656,7 @@ camel_mapi_folder_init (CamelMapiFolder *mapi_folder)
 	mapi_folder->priv = CAMEL_MAPI_FOLDER_GET_PRIVATE (mapi_folder);
 
 	folder->permanent_flags = CAMEL_MESSAGE_ANSWERED | CAMEL_MESSAGE_DELETED |
-		CAMEL_MESSAGE_DRAFT | CAMEL_MESSAGE_FLAGGED | CAMEL_MESSAGE_SEEN;
+		CAMEL_MESSAGE_DRAFT | CAMEL_MESSAGE_FLAGGED | CAMEL_MESSAGE_SEEN | CAMEL_MESSAGE_JUNK;
 
 	folder->folder_flags = CAMEL_FOLDER_HAS_SUMMARY_CAPABILITY | CAMEL_FOLDER_HAS_SEARCH_CAPABILITY;
 
