@@ -38,16 +38,13 @@
 #include "camel-mapi-store-summary.h"
 #include "camel-mapi-summary.h"
 #include "camel-mapi-notifications.h"
-
+#include "account-setup-eplugin/exchange-mapi-account-listener.h"
 #include <exchange-mapi-utils.h>
 //#define d(x) x
 
 /* This definition should be in-sync with those in exchange-mapi-account-setup.c and exchange-account-listener.c */
 #define E_PASSWORD_COMPONENT "ExchangeMAPI"
 
-#define DISPLAY_NAME_FAVOURITES _("Favorites")
-#define DISPLAY_NAME_ALL_PUBLIC_FOLDERS _("All Public Folders")
-
 #include <sys/types.h>
 #include <stdint.h>
 #include <stdbool.h>
@@ -70,6 +67,7 @@ struct _CamelMapiStorePrivate {
 
 	GHashTable *id_hash; /*get names from ids*/
 	GHashTable *name_hash;/*get ids from names*/
+	GHashTable *container_hash;
 	GHashTable *parent_hash;
 	GHashTable *default_folders; /*Default Type : Folder ID*/
 
@@ -95,13 +93,18 @@ static gboolean		mapi_delete_folder(CamelStore *, const gchar *, CamelException
 static gboolean		mapi_rename_folder(CamelStore *, const gchar *, const gchar *, CamelException *);
 static CamelFolderInfo	*mapi_get_folder_info(CamelStore *, const gchar *, guint32, CamelException *);
 static gboolean		mapi_subscribe_folder(CamelStore *, const gchar *, CamelException *);
-static gboolean mapi_folder_subscribed (CamelStore *store, const gchar *folder_name);
+static gboolean mapi_folder_is_subscribed (CamelStore *store, const gchar *folder_name);
 static gboolean		mapi_unsubscribe_folder(CamelStore *, const gchar *, CamelException *);
 static gboolean		mapi_noop(CamelStore *, CamelException *);
 static CamelFolderInfo * mapi_build_folder_info(CamelMapiStore *mapi_store, const gchar *parent_name, const gchar *folder_name);
 static gboolean mapi_fid_is_system_folder (CamelMapiStore *mapi_store, const gchar *fid);
+static void mapi_update_hash_table_type (CamelMapiStore *store, const gchar *full_name, guint *folder_type);
+static CamelFolder *mapi_get_trash (CamelStore *store, CamelException *ex);
+static CamelFolder *mapi_get_junk (CamelStore *store, CamelException *ex);
+static gboolean mapi_can_refresh_folder (CamelStore *store, CamelFolderInfo *info, CamelException *ex);
 
 static void mapi_update_folder_hash_tables (CamelMapiStore *store, const gchar *name, const gchar *fid, const gchar *parent_id);
+guint mapi_folders_hash_table_type_lookup (CamelMapiStore *store, const gchar *name);
 /* static const gchar * mapi_folders_hash_table_name_lookup (CamelMapiStore *store, const gchar *fid, gboolean use_cache); */
 #if 0
 static const gchar * mapi_folders_hash_table_fid_lookup (CamelMapiStore *store, const gchar *name, gboolean use_cache);
@@ -195,7 +198,8 @@ mapi_store_finalize (GObject *object)
 
 	if (priv->default_folders != NULL)
 		g_hash_table_destroy (priv->default_folders);
-
+	if (priv->container_hash != NULL)
+		g_hash_table_destroy (priv->container_hash);
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (camel_mapi_store_parent_class)->finalize (object);
 }
@@ -230,9 +234,12 @@ camel_mapi_store_class_init (CamelMapiStoreClass *class)
 	store_class->get_folder_info = mapi_get_folder_info;
 	store_class->free_folder_info = camel_store_free_folder_info_full;
 	store_class->subscribe_folder = mapi_subscribe_folder;
-	store_class->folder_subscribed = mapi_folder_subscribed;
+	store_class->folder_is_subscribed = mapi_folder_is_subscribed;
 	store_class->unsubscribe_folder = mapi_unsubscribe_folder;
 	store_class->noop = mapi_noop;
+	store_class->get_trash = mapi_get_trash;
+	store_class->get_junk = mapi_get_junk;
+	store_class->can_refresh_folder = mapi_can_refresh_folder;
 }
 
 /*
@@ -296,11 +303,11 @@ static gboolean mapi_construct(CamelService *service, CamelSession *session,
 	priv->name_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free); /* folder Full name to folder ID */
 	/*priv->parent_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free); / * folder ID to its parent folder ID */
 	priv->default_folders = g_hash_table_new_full (g_int_hash, g_int_equal, g_free, g_free); /* default folder type to folder ID */
-
+	priv->container_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	store->flags &= ~CAMEL_STORE_VJUNK;
 	store->flags &= ~CAMEL_STORE_VTRASH;
 
-	store->flags |= CAMEL_STORE_SUBSCRIPTIONS;
+	store->flags |= CAMEL_STORE_SUBSCRIPTIONS | CAMEL_STORE_REAL_JUNK_FOLDER;
 
 	return TRUE;
 }
@@ -335,13 +342,14 @@ mapi_auth_loop (CamelService *service, CamelException *ex)
 
 	gchar *errbuf = NULL;
 	gboolean authenticated = FALSE;
+	guint32 prompt_flags = CAMEL_SESSION_PASSWORD_SECRET;
 
 	service->url->passwd = NULL;
 
 	while (!authenticated) {
 		if (errbuf) {
 			/* We need to un-cache the password before prompting again */
-			camel_session_forget_password (session, service, E_PASSWORD_COMPONENT, "password", ex);
+			prompt_flags |= CAMEL_SESSION_PASSWORD_REPROMPT;
 			g_free (service->url->passwd);
 			service->url->passwd = NULL;
 		}
@@ -359,7 +367,7 @@ mapi_auth_loop (CamelService *service, CamelException *ex)
 						  service->url->host);
 			service->url->passwd =
 				camel_session_get_password (session, service, E_PASSWORD_COMPONENT,
-							    prompt, "password", CAMEL_SESSION_PASSWORD_SECRET, ex);
+							    prompt, "password", prompt_flags, ex);
 			g_free (prompt);
 			g_free (errbuf);
 			errbuf = NULL;
@@ -498,7 +506,7 @@ mapi_get_folder(CamelStore *store, const gchar *folder_name, guint32 flags, Came
 {
 	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
 	CamelMapiStorePrivate *priv = mapi_store->priv;
-	CamelMapiStoreInfo *si;
+	CamelStoreInfo *si;
 	CamelFolder *folder;
 	gchar *storage_path;
 
@@ -527,7 +535,7 @@ mapi_get_folder(CamelStore *store, const gchar *folder_name, guint32 flags, Came
 	}
 
 	if (si)
-		camel_store_summary_info_free ((CamelStoreSummary *)mapi_store->summary, (CamelStoreInfo *)si);
+		camel_store_summary_info_free ((CamelStoreSummary *)mapi_store->summary, si);
 
 	storage_path = g_strdup_printf ("%s/folders", priv->storage_path);
 	folder = camel_mapi_folder_new (store, folder_name, storage_path, flags, ex);
@@ -569,7 +577,7 @@ mapi_create_folder(CamelStore *store, const gchar *parent_name, const gchar *fol
 	camel_service_lock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	exchange_mapi_util_mapi_id_from_string (parent_id, &parent_fid);
-	new_folder_id = exchange_mapi_connection_create_folder (priv->conn, olFolderInbox, parent_fid, folder_name);
+	new_folder_id = exchange_mapi_connection_create_folder (priv->conn, olFolderInbox, parent_fid, 0, folder_name);
 	if (new_folder_id != 0) {
 		CamelMapiStoreInfo *si;
 		gchar *fid = g_strdup_printf ("%016" G_GINT64_MODIFIER "X", new_folder_id);
@@ -581,7 +589,7 @@ mapi_create_folder(CamelStore *store, const gchar *parent_name, const gchar *fol
 
 		mapi_update_folder_hash_tables (mapi_store, root->full_name, fid, parent_id);
 
-		camel_object_trigger_event (CAMEL_OBJECT (store), "folder_created", root);
+		camel_store_folder_created (store, root);
 	}
 
 	camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
@@ -622,7 +630,7 @@ mapi_forget_folder (CamelMapiStore *mapi_store, const gchar *folder_name, CamelE
 	camel_store_summary_save ((CamelStoreSummary *)mapi_store->summary);
 
 	fi = mapi_build_folder_info (mapi_store, NULL, folder_name);
-	camel_object_trigger_event (CAMEL_OBJECT (mapi_store), "folder_deleted", fi);
+	camel_store_folder_deleted (CAMEL_STORE (mapi_store), fi);
 	camel_folder_info_free (fi);
 }
 
@@ -645,7 +653,7 @@ mapi_delete_folder(CamelStore *store, const gchar *folder_name, CamelException *
 
 	folder_id = g_hash_table_lookup (priv->name_hash, folder_name);
 	exchange_mapi_util_mapi_id_from_string (folder_id, &folder_fid);
-	status = exchange_mapi_connection_remove_folder (priv->conn, folder_fid);
+	status = exchange_mapi_connection_remove_folder (priv->conn, folder_fid, 0);
 
 	if (status) {
 		/* Fixme ??  */
@@ -783,7 +791,7 @@ mapi_rename_folder(CamelStore *store, const gchar *old_name, const gchar *new_na
 		gchar *folder_id;
 
 		/* renaming in the same folder, thus no MoveFolder necessary */
-		if (!exchange_mapi_connection_rename_folder (priv->conn, old_fid, tmp ? tmp : new_name)) {
+		if (!exchange_mapi_connection_rename_folder (priv->conn, old_fid, 0, tmp ? tmp : new_name)) {
 			/*To translators : '%s to %s' is current name of the folder  and
 			new name of the folder.*/
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
@@ -836,7 +844,7 @@ mapi_rename_folder(CamelStore *store, const gchar *old_name, const gchar *new_na
 		} else if (!old_parent_fid_str || !new_parent_fid_str ||
 			   !exchange_mapi_util_mapi_id_from_string (old_parent_fid_str, &old_parent_fid) ||
 			   !exchange_mapi_util_mapi_id_from_string (new_parent_fid_str, &new_parent_fid) ||
-			   !exchange_mapi_connection_move_folder (priv->conn, old_fid, old_parent_fid, new_parent_fid, tmp)) {
+			   !exchange_mapi_connection_move_folder (priv->conn, old_fid, old_parent_fid, 0, new_parent_fid, 0, tmp)) {
 			camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot rename MAPI folder '%s' to '%s'"), old_name, new_name);
 			g_free (old_parent);
@@ -1034,7 +1042,7 @@ mapi_build_folder_info(CamelMapiStore *mapi_store, const gchar *parent_name, con
 	} else
 		fi->full_name = g_strdup(folder_name);
 
-	url = camel_url_new(priv->base_url,NULL);
+	url = camel_url_new (priv->base_url, NULL);
 	g_free(url->path);
 
 	url->path = g_strdup_printf("/%s", fi->full_name);
@@ -1100,8 +1108,11 @@ mapi_get_folder_info_offline (CamelStore *store, const gchar *top,
 			camel_store_summary_info_free ((CamelStoreSummary *)mapi_store->summary, si);
 			continue;
 		}
-
-		if ( !strcmp(name, camel_mapi_store_info_full_name (mapi_store->summary, si))
+		
+		if (!subscription_list && !(si->flags & CAMEL_MAPI_FOLDER_MAIL) && si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED && 
+												si->flags & CAMEL_MAPI_FOLDER_PUBLIC)
+			continue;
+		if (!strcmp(name, camel_mapi_store_info_full_name (mapi_store->summary, si))
 		     || match_path (path, camel_mapi_store_info_full_name (mapi_store->summary, si))) {
 
 			const gchar *store_info_path = camel_store_info_path((CamelStoreSummary *)mapi_store->summary, si);
@@ -1247,6 +1258,16 @@ camel_mapi_store_connected (CamelMapiStore *store, CamelException *ex)
 	return FALSE;
 }
 
+static void 
+mapi_update_hash_table_type (CamelMapiStore *store, const gchar *full_name, guint *folder_type)
+{
+	CamelMapiStorePrivate  *priv = store->priv;
+	if (full_name && folder_type) {
+		if (!g_hash_table_lookup (priv->container_hash, full_name))
+			g_hash_table_insert (priv->container_hash, g_strdup (full_name), folder_type);
+	}
+}
+
 static void
 mapi_update_folder_hash_tables (CamelMapiStore *store, const gchar *full_name, const gchar *fid, const gchar *parent_id)
 {
@@ -1286,6 +1307,16 @@ mapi_folders_update_hash_tables_from_cache (CamelMapiStore *store)
 }
 
 /* static const gchar * */
+
+guint 
+mapi_folders_hash_table_type_lookup (CamelMapiStore *store, const gchar *name) 
+{
+	CamelMapiStorePrivate  *priv = store->priv;
+	guint *folder_type = g_hash_table_lookup (priv->container_hash, name);
+	
+	return *folder_type;
+}
+
 const gchar *
 mapi_folders_hash_table_name_lookup (CamelMapiStore *store, const gchar *fid,
 				     gboolean use_cache)
@@ -1351,8 +1382,8 @@ remove_path_from_store_summary (const gchar *path, gpointer value, CamelMapiStor
 				fi->name = g_strdup (fi->name + 1);
 		}
 
-		camel_object_trigger_event (CAMEL_OBJECT (mstore), "folder_unsubscribed", fi);
-		camel_object_trigger_event (CAMEL_OBJECT (mstore), "folder_deleted", fi);
+		camel_store_folder_unsubscribed (CAMEL_STORE (mstore), fi);
+		camel_store_folder_deleted (CAMEL_STORE (mstore), fi);
 		camel_folder_info_free (fi);
 
 		camel_store_summary_info_free ((CamelStoreSummary *)mstore->summary, si);
@@ -1362,7 +1393,7 @@ remove_path_from_store_summary (const gchar *path, gpointer value, CamelMapiStor
 }
 
 static void
-mapi_folders_sync (CamelMapiStore *store, const gchar *top, guint32 flags, CamelException *ex)
+mapi_folders_sync (CamelMapiStore *store, guint32 flags, CamelException *ex)
 {
 	CamelMapiStorePrivate  *priv = store->priv;
 	gboolean status;
@@ -1396,26 +1427,21 @@ mapi_folders_sync (CamelMapiStore *store, const gchar *top, guint32 flags, Camel
 		if (si == NULL)
 			continue;
 
-		g_hash_table_insert (old_cache_folders, g_strdup (camel_store_info_path (store->summary, si)), GINT_TO_POINTER (1));
+		/* those whose left in old_cache_folders are removed at the end,
+		   which is not good for public folders, thus preserve them from
+		   an automatic removal */
+		if ((si->flags & CAMEL_MAPI_FOLDER_PUBLIC) == 0 || (si->flags & CAMEL_FOLDER_SUBSCRIBED) == 0)
+			g_hash_table_insert (old_cache_folders, g_strdup (camel_store_info_path (store->summary, si)), GINT_TO_POINTER (1));
+
 		camel_store_summary_info_free ((CamelStoreSummary *)store->summary, si);
 	}
 
 	subscription_list = (flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIPTION_LIST);
-
 	if (subscription_list) {
 		/*Consult the name <-> fid hash table for a FID.*/
-		gchar *parent_id = NULL;
-		mapi_id_t pid = 0;
-
-		parent_id = (top) ? g_strdup (camel_mapi_store_folder_id_lookup_offline (store, top)) : NULL;
-		exchange_mapi_util_mapi_id_from_string (parent_id, &pid);
-
-		status = exchange_mapi_connection_get_pf_folders_list (priv->conn, &folder_list, pid);
-
+		status = exchange_mapi_connection_get_pf_folders_list (priv->conn, &folder_list);
 		if (!status)
 			g_warning ("Could not get Public folder list..\n");
-
-		g_free (parent_id);
 	}
 
 	temp_list = folder_list;
@@ -1424,8 +1450,7 @@ mapi_folders_sync (CamelMapiStore *store, const gchar *top, guint32 flags, Camel
 	url = camel_url_to_string (CAMEL_SERVICE(store)->url,
 				   (CAMEL_URL_HIDE_PASSWORD|
 				    CAMEL_URL_HIDE_PARAMS|
-				    CAMEL_URL_HIDE_AUTH) );
-
+				    CAMEL_URL_HIDE_AUTH));
 	if ( url[strlen(url) - 1] != '/') {
 		temp_url = g_strconcat (url, "/", NULL);
 		g_free ((gchar *)url);
@@ -1434,8 +1459,9 @@ mapi_folders_sync (CamelMapiStore *store, const gchar *top, guint32 flags, Camel
 
 	/*populate the hash table for finding the mapping from container id <-> folder name*/
 	for (;temp_list != NULL; temp_list = g_slist_next (temp_list) ) {
-		const gchar *full_name;
+		const gchar *full_name = NULL;
 		gchar *fid = NULL, *parent_id = NULL, *tmp = NULL;
+		guint *folder_type = g_new0 (guint, 1);
 
 		fid = g_strdup_printf ("%016" G_GINT64_MODIFIER "X", exchange_mapi_folder_get_fid((ExchangeMAPIFolder *)(temp_list->data)));
 		parent_id = g_strdup_printf ("%016" G_GINT64_MODIFIER "X", exchange_mapi_folder_get_parent_id ((ExchangeMAPIFolder *)(temp_list->data)));
@@ -1454,16 +1480,15 @@ mapi_folders_sync (CamelMapiStore *store, const gchar *top, guint32 flags, Camel
 
 		/* remove from here; what lefts is not on the server any more */
 		g_hash_table_remove (old_cache_folders, full_name);
-
+		*folder_type = ((ExchangeMAPIFolder *)(temp_list->data))->container_class;
 		mapi_update_folder_hash_tables (store, full_name, fid, parent_id);
-
+		mapi_update_hash_table_type (store, full_name, folder_type);
 		if (((ExchangeMAPIFolder *)(temp_list->data))->is_default) {
 			guint *type = g_new0 (guint, 1);
 			*type = ((ExchangeMAPIFolder *)(temp_list->data))->default_type;
 			g_hash_table_insert (priv->default_folders, type,
 					     g_strdup(fid));
 		}
-
 		g_free (fid);
 		g_free (parent_id);
 		g_free (tmp);
@@ -1475,42 +1500,61 @@ mapi_folders_sync (CamelMapiStore *store, const gchar *top, guint32 flags, Camel
 		if (folder->default_type == olPublicFoldersAllPublicFolders)
 			continue;
 
-		if ( folder->container_class != MAPI_FOLDER_TYPE_MAIL)
-			continue;
-
-		info = mapi_convert_to_folder_info (store, folder, (const gchar *)url, ex);
-		if (!(mapi_si = (CamelMapiStoreInfo *) camel_store_summary_path ((CamelStoreSummary *)store->summary, info->full_name))) {
+		if ( folder->container_class == MAPI_FOLDER_TYPE_MAIL) {
+			info = mapi_convert_to_folder_info (store, folder, (const gchar *)url, ex);
+			info->flags |= CAMEL_MAPI_FOLDER_MAIL;
+			mapi_si = (CamelMapiStoreInfo *) camel_store_summary_path ((CamelStoreSummary *)store->summary, info->full_name);
+			
+			if (!mapi_si) {
+				gchar *fid, *pfid = NULL;
+	
+				fid = g_strdup_printf ("%016" G_GINT64_MODIFIER "X",
+						       exchange_mapi_folder_get_fid((ExchangeMAPIFolder *)(folder_list->data)));
+				pfid = g_strdup_printf ("%016" G_GINT64_MODIFIER "X",
+							exchange_mapi_folder_get_parent_id((ExchangeMAPIFolder *)(folder_list->data)));
+	
+				mapi_si = camel_mapi_store_summary_add_from_full (store->summary, info->full_name, '/', fid, pfid);
+				g_free (fid);
+				g_free (pfid);
+				if (mapi_si == NULL)
+					continue;
+	
+				camel_store_summary_info_ref ((CamelStoreSummary *)store->summary, (CamelStoreInfo *)mapi_si);
+	
+				if (!subscription_list) {
+					camel_store_folder_created (CAMEL_STORE (store), info);
+					camel_store_folder_subscribed (CAMEL_STORE (store), info);
+				}
+			}
+		
+			mapi_si->info.flags |= info->flags;
+			mapi_si->info.total = info->total;
+			mapi_si->info.unread = info->unread;
+			
+			camel_store_summary_info_free ((CamelStoreSummary *)store->summary, (CamelStoreInfo *)mapi_si);
+			camel_folder_info_free (info);
+		} else if (folder->category == MAPI_FAVOURITE_FOLDER) {
 			gchar *fid, *pfid = NULL;
-
+			info = mapi_convert_to_folder_info (store, folder, (const gchar *)url, ex);
+			mapi_si = (CamelMapiStoreInfo *) camel_store_summary_path ((CamelStoreSummary *)store->summary, info->full_name);
 			fid = g_strdup_printf ("%016" G_GINT64_MODIFIER "X",
-					       exchange_mapi_folder_get_fid((ExchangeMAPIFolder *)(folder_list->data)));
+						exchange_mapi_folder_get_fid((ExchangeMAPIFolder *)(folder_list->data)));
 			pfid = g_strdup_printf ("%016" G_GINT64_MODIFIER "X",
 						exchange_mapi_folder_get_parent_id((ExchangeMAPIFolder *)(folder_list->data)));
-
 			mapi_si = camel_mapi_store_summary_add_from_full (store->summary, info->full_name, '/', fid, pfid);
-
 			g_free (fid);
 			g_free (pfid);
-			if (mapi_si == NULL) {
+
+			if (mapi_si == NULL)
 				continue;
-			}
 
 			camel_store_summary_info_ref ((CamelStoreSummary *)store->summary, (CamelStoreInfo *)mapi_si);
-
-			if (!subscription_list) {
-				camel_object_trigger_event (CAMEL_OBJECT (store), "folder_created", info);
-				camel_object_trigger_event (CAMEL_OBJECT (store), "folder_subscribed", info);
-			}
+			mapi_si->info.flags |= info->flags;
+			camel_store_summary_info_free ((CamelStoreSummary *)store->summary, (CamelStoreInfo *)mapi_si);
+			camel_folder_info_free (info);	
 		}
-
-		mapi_si->info.flags |= info->flags;
-		mapi_si->info.total = info->total;
-		mapi_si->info.unread = info->unread;
-
-		camel_store_summary_info_free ((CamelStoreSummary *)store->summary, (CamelStoreInfo *)mapi_si);
-		camel_folder_info_free (info);
 	}
-
+	
 	/* Weed out deleted folders */
 	g_hash_table_foreach (old_cache_folders, (GHFunc) remove_path_from_store_summary, store);
 	g_hash_table_destroy (old_cache_folders);
@@ -1541,7 +1585,6 @@ mapi_get_folder_info(CamelStore *store, const gchar *top, guint32 flags, CamelEx
 {
 	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
 	CamelFolderInfo *info = NULL;
-	gint s_count = 0;
 
 	/*
 	 * Thanks to Michael, for his cached folders implementation in IMAP
@@ -1565,7 +1608,7 @@ mapi_get_folder_info(CamelStore *store, const gchar *top, guint32 flags, CamelEx
 	     (!mapi_store->priv->folders_synced) ||
 	     (top && *top && !camel_mapi_store_folder_id_lookup (mapi_store, top))) &&
 	    (check_for_connection ((CamelService *)store, ex) || ((CamelService *)store)->status == CAMEL_SERVICE_CONNECTING)) {
-		mapi_folders_sync (mapi_store, top, flags, ex);
+		mapi_folders_sync (mapi_store, flags, ex);
 
 		if (camel_exception_is_set (ex)) {
 			camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
@@ -1577,7 +1620,6 @@ mapi_get_folder_info(CamelStore *store, const gchar *top, guint32 flags, CamelEx
 
 	camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
-	s_count = camel_store_summary_count((CamelStoreSummary *)mapi_store->summary);
 	info = mapi_get_folder_info_offline (store, top, flags, ex);
 	return info;
 }
@@ -1636,47 +1678,67 @@ static gboolean
 mapi_subscribe_folder(CamelStore *store, const gchar *folder_name, CamelException *ex)
 {
 	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
-
+	guint folder_type = mapi_folders_hash_table_type_lookup(mapi_store, folder_name);
 	CamelFolderInfo *fi;
 	CamelStoreInfo *si = NULL;
-	gchar *parent_name = NULL;
-	gchar *f_name = NULL;
+	const gchar *parent_name = NULL, *use_folder_name = folder_name, *fid = NULL;
+	gboolean favourites = FALSE;
 	/* TODO : exchange_mapi_add_to_favorites (); */
+	
+	fid = camel_mapi_store_folder_id_lookup(mapi_store, folder_name);
 
+	if (g_str_has_prefix (folder_name, DISPLAY_NAME_ALL_PUBLIC_FOLDERS) ) {
+		const gchar *f_name = NULL;
+
+		parent_name = DISPLAY_NAME_FAVOURITES;
+
+		f_name = strrchr (folder_name,'/');
+		if (!f_name) {
+			/* Don't process All Public Folder. */
+			return TRUE;
+		}
+
+		use_folder_name = ++f_name;
+		favourites = TRUE;
+	}
 	si = camel_store_summary_path((CamelStoreSummary *)mapi_store->summary, folder_name);
+
 	if (si != NULL) {
 		if ((si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED) == 0) {
 			si->flags |= CAMEL_STORE_INFO_FOLDER_SUBSCRIBED;
 			si->flags |= CAMEL_FOLDER_SUBSCRIBED;
 			camel_store_summary_touch((CamelStoreSummary *)mapi_store->summary);
 		}
-		camel_store_summary_info_free((CamelStoreSummary *)mapi_store->summary, si);
 	}
 
-	if (g_str_has_prefix (folder_name, DISPLAY_NAME_ALL_PUBLIC_FOLDERS) ) {
-		parent_name = DISPLAY_NAME_FAVOURITES;
-
-		f_name = strrchr(folder_name,'/');
-		if (f_name != NULL)
-			folder_name = ++f_name;
-		else  //Don't process All Public Folder.
-			return TRUE;
-	}
-
-	fi = mapi_build_folder_info (mapi_store, parent_name, folder_name);
-
-	fi->flags |= CAMEL_FOLDER_SUBSCRIBED;
-	fi->flags |= CAMEL_FOLDER_NOCHILDREN;
-	fi->flags |= CAMEL_STORE_INFO_FOLDER_SUBSCRIBED;
-
-	camel_object_trigger_event (CAMEL_OBJECT (store), "folder_subscribed", fi);
-	camel_folder_info_free (fi);
+	if (folder_type == MAPI_FOLDER_TYPE_MAIL || folder_type == MAPI_FOLDER_TYPE_UNKNOWN) {
+		fi = mapi_build_folder_info (mapi_store, parent_name, use_folder_name);		
+		if (favourites) {
+			CamelURL *url;
+			url = camel_url_new (mapi_store->priv->base_url, NULL);
+			url->path = g_strdup_printf ("/%s", camel_store_info_path (mapi_store->summary, si));
+			g_free (fi->uri);
+			fi->uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
+			camel_url_free (url);
+		}
 
+		fi->unread = si->unread;
+		fi->total = si->total;
+		fi->flags = si->flags;
+		fi->flags |= CAMEL_FOLDER_SUBSCRIBED;
+		fi->flags |= CAMEL_FOLDER_NOCHILDREN;
+		fi->flags |= CAMEL_STORE_INFO_FOLDER_SUBSCRIBED;
+		camel_store_folder_subscribed (store, fi);
+		camel_folder_info_free (fi);
+	} else
+		exchange_mapi_add_esource (CAMEL_SERVICE(mapi_store)->url, use_folder_name, fid, folder_type);
+	
+	camel_store_summary_info_free((CamelStoreSummary *)mapi_store->summary, si);
 	return TRUE;
 }
 
 static gboolean
-mapi_folder_subscribed (CamelStore *store, const gchar *folder_name)
+mapi_folder_is_subscribed (CamelStore *store, const gchar *folder_name)
 {
 	CamelMapiStore *mapi_store = (CamelMapiStore *) store;
 	CamelStoreInfo *si;
@@ -1696,10 +1758,13 @@ mapi_unsubscribe_folder(CamelStore *store, const gchar *folder_name, CamelExcept
 	CamelFolderInfo *fi;
 	CamelStoreInfo *si;
 	gchar *parent_name = NULL;
+	const gchar *fid = NULL;
 	gchar *f_name = NULL;
 
 	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
-
+	CamelURL *url = CAMEL_SERVICE (mapi_store)->url;
+	guint folder_type = mapi_folders_hash_table_type_lookup(mapi_store, folder_name);
+	fid = camel_mapi_store_folder_id_lookup(mapi_store, folder_name);
 	si = camel_store_summary_path((CamelStoreSummary *)mapi_store->summary, folder_name);
 	if (si) {
 		if (si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED) {
@@ -1707,7 +1772,6 @@ mapi_unsubscribe_folder(CamelStore *store, const gchar *folder_name, CamelExcept
 			camel_store_summary_touch((CamelStoreSummary *)mapi_store->summary);
 			camel_store_summary_save((CamelStoreSummary *)mapi_store->summary);
 		}
-		camel_store_summary_info_free((CamelStoreSummary *)mapi_store->summary, si);
 	}
 
 	if (g_str_has_prefix (folder_name, DISPLAY_NAME_ALL_PUBLIC_FOLDERS) ) {
@@ -1719,12 +1783,15 @@ mapi_unsubscribe_folder(CamelStore *store, const gchar *folder_name, CamelExcept
 		else //Don't process All Public Folder.
 			return TRUE;
 	}
-
-	fi = mapi_build_folder_info (mapi_store, parent_name, folder_name);
-
-	camel_object_trigger_event (CAMEL_OBJECT (store), "folder_unsubscribed", fi);
-	camel_folder_info_free (fi);
-
+	if (si->flags & CAMEL_MAPI_FOLDER_MAIL) {
+		fi = mapi_build_folder_info (mapi_store, parent_name, folder_name);
+		camel_store_folder_unsubscribed (store, fi);
+		camel_folder_info_free (fi);
+	} else 
+		exchange_mapi_remove_esource(url, folder_name, fid, folder_type);
+	
+	camel_store_summary_info_free((CamelStoreSummary *)mapi_store->summary, si);
+		
 	return TRUE;
 }
 
@@ -1743,3 +1810,67 @@ camel_mapi_store_get_exchange_connection (CamelMapiStore *mapi_store)
 
 	return mapi_store->priv->conn;
 }
+
+static CamelFolder *
+mapi_get_folder_with_type (CamelStore *store, guint folder_type, CamelException *ex)
+{
+	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
+	CamelFolderInfo *all_fi, *fi;
+	CamelFolder *folder = NULL;
+
+	g_return_val_if_fail (mapi_store != NULL, NULL);
+	g_return_val_if_fail (mapi_store->priv != NULL, NULL);
+
+	all_fi = camel_store_get_folder_info (store, NULL, CAMEL_STORE_FOLDER_INFO_RECURSIVE, ex);
+	fi = all_fi;
+	while (fi) {
+		CamelFolderInfo *next;
+
+		if ((fi->flags & CAMEL_FOLDER_TYPE_MASK) == folder_type) {
+			folder = camel_store_get_folder (store, fi->full_name, 0, ex);
+			break;
+		}
+
+		/* move to the next, depth-first search */
+		next = fi->child;
+		if (!next)
+			next = fi->next;
+		if (!next) {
+			next = fi->parent;
+			while (next) {
+				CamelFolderInfo *sibl = next->next;
+				if (sibl) {
+					next = sibl;
+					break;
+				} else {
+					next = next->parent;
+				}
+			}
+		}
+
+		fi = next;
+	}
+
+	camel_store_free_folder_info (store, all_fi);
+
+	return folder;
+}
+
+static CamelFolder *
+mapi_get_trash (CamelStore *store, CamelException *ex)
+{
+	return mapi_get_folder_with_type (store, CAMEL_FOLDER_TYPE_TRASH, ex);
+}
+
+static CamelFolder *
+mapi_get_junk (CamelStore *store, CamelException *ex)
+{
+	return mapi_get_folder_with_type (store, CAMEL_FOLDER_TYPE_JUNK, ex);
+}
+
+static gboolean
+mapi_can_refresh_folder (CamelStore *store, CamelFolderInfo *info, CamelException *ex)
+{
+	return CAMEL_STORE_CLASS(camel_mapi_store_parent_class)->can_refresh_folder (store, info, ex) ||
+	      (camel_url_get_param (((CamelService *)store)->url, "check_all") != NULL);
+}
